%%% Function to estimate manifold signal components from
%%% compressive measurements (1D case)
%%% Chin, March 2012

%%% Paper: "Signal Recovery on Incoherent Manifolds"
%%% Authors: Chinmay Hegde and Richard G. Baraniuk
%%% Submitted to IEEE Trans. Info. Theory, Jan 2012

%%% INPUTS: y: Mx1 measurement vector
%%%         Phi: MxN measurement matrix
%%%         recopts: additional parameters
%%% OUTPUTS: xhat, ahat, bhat: signal estimates.
%%%          xhat = ahat + bhat
%%% SUBROUTINES: proj: performs 1D matched filtering
%%%              thresh: performs K-sparse approximation

function [xhat, ahat,bhat] = spinrec_1d(y, Phi,recopts)
stp = recopts.stp;
iter = recopts.iter;
x0 = recopts.base;
xhat = recopts.init;

ahat = 0*xhat; bhat = 0*xhat;
xhat = ahat + bhat;
xhat_prev = xhat;

K = recopts.K;
converr = 1;
kk = 1;

while kk < iter && converr > 1e-10
    
    r = y - Phi*xhat_prev;
    
    s1 = ahat + stp*(Phi'*r);
    s2 = bhat + stp*(Phi'*r);
    
    ahat = proj(s1,x0,recopts);
    bhat = thresh(s2,K);
    
    xhat = ahat + bhat;
    
    kk = kk+1;
    converr = norm(xhat-xhat_prev)/norm(xhat);
    xhat_prev = xhat;

end

end


% manifold projection function (match filter with template x0)
function xhat = proj(z,x0,~)
c = xcorr(z,x0);
[~,T0] = max(c);
T0 = T0 -length(x0);
xhat = circshift(x0,T0);
end

% sparse projection function
function zhat = thresh(z,K)
zhat = 0*z;
[~,idx] = sort(abs(z),'descend');
zhat(idx(1:K)) = z(idx(1:K));
end